Function SetupAudio()
    audioOutput = CreateObject("roAssociativeArray")
    audioOutput["HDMI"] = CreateObject("roAudioOutput", "HDMI")
    audioOutput["Analog"] = CreateObject("roAudioOutput", "Analog")
    audioOutput["SPDIF"] = CreateObject("roAudioOutput", "SPDIF")
    audioOutput["USB"] = CreateObject("roAudioOutput", "USB")
    audioOutput["NONE"] = CreateObject("roAudioOutput", "NONE")

    print "SetupAudio completed"
    return audioOutput
End Function

Function SetupDebug()
    debugRegistrySection = CreateObject("roRegistrySection", "debug")

    if debugRegistrySection.Read("enabled") = "1" then

        ' Native Debug for firmware higher then 8.5
        deviceInfo = CreateObject("roDeviceInfo")
        if deviceInfo.FirmwareIsAtLeast("8.5.31") then
            registryHtmlSection = CreateObject("roRegistrySection", "html")
            registryHtmlSection.write("enable_web_inspector", "1")
            registryHtmlSection.flush()
        endif

        networkingRegistry = CreateObject("roRegistrySection", "networking")
        ' access to BrightSign web interface
        networkingRegistry.write("http_server", "80")
        networkingRegistry.write("ssh","22")
        networkingRegistry.write("telnet","23")
        networkingRegistry.flush()
    endif

    print "SetupDebug completed"
    return debugRegistrySection
End Function

Function SetupVideoMode()
    videoModeProvider = CreateObject("roVideoMode")
    videoModeProvider.SetGraphicsZOrder("back")
    videoModeProvider.SetBackgroundColor(&h18182c)

    print "SetupVideoMode completed"
    return videoModeProvider
End Function

Function GetSupportedVideoModes(videoModeProvider)
    allModes = videoModeProvider.GetAvailableModes()
    supportedModes = CreateObject("roArray", 0, true)
    includedModes = CreateObject("roAssociativeArray")
    While allModes.IsNext()
        mode = allModes.Next()
        modeKey = CreateObject("roString")
        width = mode["width"].ToStr()
        height = mode["height"].ToStr()
        framerate = mode["framerate"].ToStr()
        modeKey.AppendString(width, width.Len())
        modeKey.AppendString("x", 1)
        modeKey.AppendString(height, height.Len())
        modeKey.AppendString("@", 1)
        modeKey.AppendString(framerate, framerate.Len())
        ' Videos in PORTRAIT orientation can not be played in interlaced modes
        if mode.interlaced = false and not(includedModes.DoesExist(modeKey)) then
            includedModes.AddReplace(modeKey, modeKey)
            supportedModes.Push(mode)
        endif
    End While
    print "supportedVideoModes";FormatJson(supportedModes, 1)
    return supportedModes
End Function

Function IsMounted(path as String)
    return CreateObject("roStorageHotplug").GetStorageStatus(path).mounted
End Function

Function GetFirstAvaliableStorageUnit()
    destinationPaths = ["SSD:", "SD:", "USB1:"]
    avaliableDestinatons = CreateObject("roArray", 3, false)
    for each destination in destinationPaths
        if IsMounted(destination) then
            avaliableDestinatons.Push(destination)
        end if
    next
    return avaliableDestinatons[0]
End function

Function GetCurrentTransform(screenRegistrySection)
    orientation = screenRegistrySection.Read("orientation")
    if orientation = "LANDSCAPE" then
        return "identity"
    endif
    if orientation = "PORTRAIT" then
        return "rot90"
    endif
    if orientation = "PORTRAIT_FLIPPED" then
        return "rot270"
    endif
    return "identity"
End Function

Function SetupHtmlWidget(msgPort, screenRegistrySection, screenRectangle, debugRegistrySection, htmlFileName, organizationUid, browserWindows)

    currentTransform = GetCurrentTransform(screenRegistrySection)
    firstAvaiableStorage = GetFirstAvaliableStorageUnit()

    config = {
        nodejs_enabled: true,
        brightsign_js_objects_enabled: true,
        mouse_enabled: true,
        focus_enabled: true,
        javascript_enabled: true,
        url: "file:///" + htmlFileName,
        storage_path: firstAvaiableStorage,
        storage_quota: 1073741824,
        transform: currentTransform,
        hwz_default: "on",
        pinch_to_zoom_enabled: true,
        scrollbar_enabled: true,
        security_params: {
            websecurity: false
        }
        'port: msgPort ' Doesn't work in 6.2 FW. Used this htmlWidget.SetPort(msgPort)
    }

    ' adding both for backwards compatibility
    if debugRegistrySection.Read("enabled") = "1" then
        debugPort = 3000
        config["inspector_server"] = {
            port: debugPort
        }
    endif

    ' ever since BrightOS 8.2 there is a mandatory requirement to disable web security by this option
    ' With BOS 8.2 and later (Chromium69 and later), several new CORS checks have been added: 
    ' security_params.websecurity or enableSecurity(false) do not disable all these checks.
    htmlRegistrySection = CreateObject("roRegistrySection", "html")
    disableWebSecurity = htmlRegistrySection.read("disable-web-security")
    if disableWebSecurity <> "1" then
        htmlRegistrySection.write("disable-web-security", "1")
        htmlRegistrySection.flush()
    endif

    ' using htmlWidget to limit possible conflict with 3rd party htmlWidget created in custom.brs
    htmlWidget = CreateObject("roHtmlWidget", screenRectangle, config)
    htmlWidget.SetPort(msgPort)

    proxyServer = GetProxyServer()
    proxyServerBypass = GetProxyBypass()

    htmlWidget.SetProxy(proxyServer)
    htmlWidget.SetProxyBypass(proxyServerBypass)
    
    ' wait for the ready message from html widget
    WaitHtmlWidgetReady(msgPort)
    htmlWidget.Show()

    htmlWidget.PostJSMessage({
        type: "ready",
        organization_uid: organizationUid
    })

    print "SetupHtmlWidget completed"
    browserWindows.AddReplace("main", htmlWidget)
    return htmlWidget
End Function

Sub WaitHtmlWidgetReady(msgPort)
    while true
        msg = Wait(0, msgPort)
        if type(msg) = "roHtmlWidgetEvent" then
            eventData = msg.GetData()
            if type(eventData) = "roAssociativeArray" and type(eventData.reason) = "roString" then
                if eventData.reason = "message" and eventData.message.type = "ready" then
                    print "htmlWidget ready"
                    exit while
                endif
            endif
        endif
    end while
End Sub

Function GetProxyServer()
    ' These Proxy helpers are used to configure networking of the roHtmlWidget
    ' roHtmlWidget ignores the general settings and configuration of roNetworkConfiguration

    ' Currently we hardcode proxy setup only for Ethernet as it's unclear how to get "current network" (eth, wifi)
    ' 0 = lan0
    ' 1 = wlan0
    networkConfiguration = CreateObject("roNetworkConfiguration", 0)
    proxyServer = networkConfiguration.getProxy()
    return proxyServer
End Function

Function GetProxyBypass()
    ' These Proxy helpers are used to configure networking of the roHtmlWidget
    ' roHtmlWidget ignores the general settings and configuration of roNetworkConfiguration

    ' Currently we hardcode proxy setup only for Ethernet as it's unclear how to get "current network" (eth, wifi)
    ' 0 = lan0
    ' 1 = wlan0
    networkConfiguration = CreateObject("roNetworkConfiguration", 0)
    proxyServerBypass = networkConfiguration.GetProxyBypass()
    return proxyServerBypass
End Function
