Function LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    metadata = videoMetadata.Lookup(videoPlayerKey)
    if metadata = Invalid then
        return CreateObject("roAssociativeArray")
    endif
    return metadata
End Function

Sub SetMetadataType(videoMetadata, videoPlayerKey, metadataType)
    metadata = LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    metadata["type"] = metadataType
    videoMetadata.AddReplace(videoPlayerKey, metadata)
End Sub

Sub SetVideoMetadataEnded(videoMetadata, videoPlayerKey, ended)
    ' used for flaging video hit time code for the first time
    metadata = LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    metadata["ended"] = ended
    videoMetadata.AddReplace(videoPlayerKey, metadata)
End Sub

Sub SetVideoMetadata(videoPlayers, videoMetadata, videoPlayerKey, isLooped)
    metadata = LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    if metadata["type"] = "video" then
        videoPlayer = videoPlayers.Lookup(videoPlayerKey)
        metadata["timer"] = CreateObject("roTimeSpan")
        metadata["duration"] = videoPlayer.GetDuration()
        metadata["stopped"] = false
        metadata["looped"] = isLooped
        videoMetadata.AddReplace(videoPlayerKey, metadata)
    endif
End Sub

Sub SetMetadataInvocationUid(videoMetadata, videoPlayerKey, isLooped, invocationUid)
    metadata = LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    metadata["invocationUid"] = invocationUid
    metadata["looped"] = isLooped
    videoMetadata.AddReplace(videoPlayerKey, metadata)
End Sub

Sub InvokeJSMessage(htmlWidget, invocationUid, result, response)
    if response = invalid then
        response = CreateObject("roAssociativeArray")
    end if
    response["type"] = "response"
    response["succeeded"] = result
    response["invocationUid"] = invocationUid
    htmlWidget.PostJSMessage(response)
    response = invalid
End Sub

Sub SetMetadataStopped(videoMetadata, videoPlayerKey)
    metadata = LoadOrCreateMetadata(videoMetadata, videoPlayerKey)
    metadata["stopped"] = true
    metadata["looped"] = false
    metadata["ended"] = false
    videoMetadata.AddReplace(videoPlayerKey, metadata)
End Sub

Sub AppendEndedEvent(videoPlayer)
    videoDuration = videoPlayer.GetDuration()
    videoPosition = videoPlayer.GetPlaybackPosition()
    remainingTime = videoDuration - videoPosition
    ' start the progressive video end tracking 1 second before the video ends
    videoPlayer.AddEvent(8, remainingTime - 1000)
End Sub

Sub PrepareMedia(screenRegistrySection, videoModeProvider, videoPlayer, event)          
    x = strtoi(event.message.x)
    y = strtoi(event.message.y)
    width = strtoi(event.message.width)
    height = strtoi(event.message.height)
    orientation = GetCurrentVideoTransform(screenRegistrySection)
    rectangle = CreateObject("roRectangle", x, y, width, height)
    videoPlayer.SetRectangle(rectangle)
    videoPlayer.SetTransform(orientation)

    ' Loop mode needs to be set before video prepare otherwise loop mode won't work even
    ' if "GetPropertiesString" method will return right value
    ' We use LoopButNotSeamless to avoid video freezes by looping FHD/UHD/HDREADY videos
    videoPlayer.SetLoopMode("LoopButNotSeamless")

    if event.message.background = "true" then
        videoModeProvider.SetGraphicsZOrder("front")
    else 
        videoModeProvider.SetGraphicsZOrder("back")
    endif
End Sub

Sub PlayVideo(videoPlayer, filePath, volume)
    videoPlayer.SetVolume(volume)
    videoPlayer.PlayFile(filePath)
End Sub

Sub PrepareStream(videoPlayer, uri)
    stream = CreateObject("roRtspStream", uri)

    print "PreloadFile:Stream"

    videoPlayer.PreloadFile({ rtsp: stream })
End Sub

Sub PlayStream(videoPlayer, uri)
    stream = CreateObject("roRtspStream", uri)
    videoPlayer.PlayFile({ rtsp: stream })
End Sub

Sub PlayHDMI(videoPlayer)
    videoInput = CreateObject("roVideoInput")
    videoPlayer.PlayFile(videoInput)
End Sub

Sub InitVideoMode(screenRegistrySection, supportedVideoModes, defaultWidth, defaultHeight)
    if not(screenRegistrySection.Exists("video_mode")) then
        SaveVideoModeIfSupported(screenRegistrySection, supportedVideoModes, defaultWidth, defaultHeight, "auto")
    endif
End Sub

Function SaveVideoModeIfSupported(screenRegistrySection, supportedVideoModes, width, height, framerate)
    supportedVideoModes.Reset()
    While supportedVideoModes.IsNext()
        mode = supportedVideoModes.Next()
        if (framerate = "auto" OR mode["framerate"] = strtoi(framerate)) AND mode["width"] = strtoi(width) AND mode["height"] = strtoi(height) then
            screenRegistrySection.Write("video_mode", FormatJson(mode, 1))
            screenRegistrySection.Flush()
            return true
        endif
    End While
    return false
End Function

Sub ApplySavedVideoMode(screenRegistrySection, videoModeProvider, fullScreenRectangle)
    if screenRegistrySection.Exists("video_mode") then
        videoMode = ParseJson(screenRegistrySection.Read("video_mode"))
        width = videoMode["width"]
        height = videoMode["height"]
        fullScreenRectangle.SetWidth(width)
        fullScreenRectangle.SetHeight(height)
        if width > 1920 then
            videoModeProvider.SetMode(videoMode["videomode"] + ":fullres") 
        else
            videoModeProvider.SetMode(videoMode["videomode"])
        end if
        print "videoMode=";screenRegistrySection.Read("video_mode")
    endif

    if screenRegistrySection.Exists("orientation") then
        print "orientation=";screenRegistrySection.Read("orientation")
    endif

    if screenRegistrySection.Exists("video_orientation") then
        print "video orientation=";screenRegistrySection.Read("video_orientation")
    endif
End Sub

Function GetCurrentVideoTransform(screenRegistrySection)
    if screenRegistrySection.Exists("video_orientation") then
        return "identity" ' LANDSCAPE_FLIPPED is not supported on this platform
    endif
    return GetCurrentTransform(screenRegistrySection)
End Function

' Backward functionality with filesystem integration
Function GetFallbackPath(path as String)
    if path = "SD:" then
        return "/storage/sd"
    endif
    if path = "SSD:" then
        return "/storage/ssd"
    endif
    if path = "USB1:" then
        return "/storage/usb1"
    endif
    return "/storage/sd" 'default
End Function

Sub EnableTouchScreen(msgPort, screenRegistrySection)
    if screenRegistrySection.Exists("video_mode") then
        touchVideoMode = ParseJson(screenRegistrySection.Read("video_mode"))
        touchWidth = touchVideoMode["graphicsPlaneWidth"]
        touchHeight = touchVideoMode["graphicsPlaneHeight"]
        touchScreen = CreateObject("roTouchScreen")
        touchScreen.SetPort(msgPort)
        touchScreen.SetResolution(touchWidth, touchHeight)
    endif
    print "EnableTouchScreen completed"
End Sub

Sub SetupOnScreenKeyboard(msgPort, screenRegistrySection, onScreenKeyboardObject)
	deviceInfo = CreateObject("roDeviceInfo")
    if deviceInfo.GetVersion() = "8.5.53" OR deviceInfo.GetVersion() = "8.5.53.2" OR deviceInfo.GetVersion() = "8.5.60" then
        onScreenKeyboardObject.Clear()
        print "SOS SetupOSK"
        currentTransform = GetCurrentTransform(screenRegistrySection)
        ' This is ready for the moment BrightSign fixes the events for virtual keyboard
        ' Until then we have to use fullscreen keyboard rectangle
        'if currentTransform = "identity" then
        '    rv=createObject("roRectangle", 480,746,960,334)
        'else if currentTransform = "rot90"
        '    rv=createObject("roRectangle", 1586,0,334,1080)
        'else if currentTransform = "rot270" then
        '    rv=createObject("roRectangle", 0,0,334,1080)
        'else
        '    rv=createObject("roRectangle", 0,0,1920,1080)
        'endif

        rv=createObject("roRectangle", 0,0,1920,1080)
        osk=createObject("roVirtualKeyboard",rv)
        osk.setTransform(currentTransform)
        osk.setResource("file:///osk/bsvirtualkb.html")
        osk.SetPort(msgPort)
        onScreenKeyboardObject.AddReplace("onScreenKeyboard", osk)
    end if
End Sub

Function SetupVideoCanvasAndResolution(videoModeProvider, screenRegistrySection, supportedVideoModes)
    deviceInfo = CreateObject("roDeviceInfo")
    model = deviceInfo.GetModel()
    print "model=";model
    hsRegex = CreateObject("roRegex", "^hs\w+$", "i")
    xtPPRegex = CreateObject("roRegex", "^xt1144-pp$", "i")
    if hsRegex.IsMatch(model) then
        print "HSxxxx series"
        defaultResolution = { width: 1280, height: 800 }
        InitVideoMode(screenRegistrySection, supportedVideoModes, "1280", "800")
    else if xtPPRegex.IsMatch(model) then
        print "XT1144-PP series"
        defaultResolution = { width: 1920, height: 1080 }
        InitVideoMode(screenRegistrySection, supportedVideoModes, "1920", "1080")
    else
        print "Any specific model hasn't been recognized"
        defaultResolution = { width: 800, height: 600 }
        InitVideoMode(screenRegistrySection, supportedVideoModes, "800", "600")
    endif

    fullScreenRectangle = CreateObject("roRectangle", 0, 0, defaultResolution.width, defaultResolution.height)
    ApplySavedVideoMode(screenRegistrySection, videoModeProvider, fullScreenRectangle)

    print "SetupVideoCanvasAndResolution completed"
    return fullScreenRectangle
End Function

Sub FirstRunSetup(systemTime)

    firstStartSection = CreateObject("roRegistrySection", "firstStart")
    if not(firstStartSection.Exists("time")) then
        print "This is first start"
        currentLocalTime = systemTime.GetLocalDateTime()
        firstStartSection.Write("time", currentLocalTime.GetString())
        ' Hook to custom.brs
        OnFirstStartHook()
    endif

    print "First setup completed"
End Sub

Function FormatDNSServersArray(dnsServersArray)
    ' Initialize an empty roArray object to hold the DNS server IPs
    dnsArray = CreateObject("roArray", 0, true)

    ' transform roAssocArray to roArray supported by SetDNSServers()
    for each key in dnsServersArray
        dnsServer = dnsServersArray.Lookup(key)
        if dnsServer <> invalid then
            dnsArray.Push(dnsServer)
        end if
    end for

    ' Check if the number of DNS servers exceeds the allowed limit
    if dnsArray.Count() > 3 then
        print "Error: More than 3 DNS servers are not allowed."
        return invalid
    end if

    ' Return the formatted DNS array
    return dnsArray
End Function

Function SetupBrowserWidget(msgPort, screenRegistrySection, htmlPath, x, y, width, height, browserWindows)

    currentTransform = GetCurrentTransform(screenRegistrySection)
    firstAvaiableStorage = GetFirstAvaliableStorageUnit()

    config = {
        nodejs_enabled: true,
        brightsign_js_objects_enabled: true,
        mouse_enabled: true,
        focus_enabled: true,
        javascript_enabled: true,
        url: htmlPath,
        user_stylesheet: firstAvaiableStorage + "/widget.css",
        transform: currentTransform,
        hwz_default: "on",
        pinch_to_zoom_enabled: true,
        scrollbar_enabled: true,
        security_params: {
            websecurity: false
        }
    }
    ' tmp disabled, probably bug in FW
    

    screenRectangle = CreateObject("roRectangle", strtoi(x), strtoi(y), strtoi(width), strtoi(height))

    ' using htmlWidget to limit possible conflict with 3rd party htmlWidget created in custom.brs
    browserWidget = CreateObject("roHtmlWidget", screenRectangle, config)
    browserWidget.SetPort(msgPort)

    proxyServer = GetProxyServer()
    proxyServerBypass = GetProxyBypass()

    browserWidget.SetProxy(proxyServer)
    browserWidget.SetProxyBypass(proxyServerBypass)

    browserWidget.Show()
    print "SetupBrowserWidget completed"
    browserWindows.AddReplace("browser", browserWidget)
    browserWindows.AddReplace("browser_x", x)
    browserWindows.AddReplace("browser_y", y)
    browserWindows.AddReplace("browser_width", width)
    browserWindows.AddReplace("browser_height", height)
    return browserWidget
End Function

Function DiscardBrowserWidget(browserWindows)

    browserWidget = browserWindows.Lookup("browser")
    if browserWidget <> invalid then
        browserWidget.Hide()
        browserWidget = invalid

        print "DiscardBrowserWidget completed"
        browserWindows.Delete("browser")
    end if

    return true
End Function
